export const getState = (state = {}, keyPath, defaultValue) => {
    if (typeof keyPath === 'string' || typeof keyPath === 'number') {
        keyPath = [keyPath];
    }
    const key = keyPath.shift();
    let subState = state[key] ?? {};
    if (keyPath.length === 0) {
        return state[key] ?? defaultValue;
    }
    if (typeof subState !== 'object') {
        subState = {};
    }
    return getState(subState, keyPath, defaultValue);
};
export const getStateIn = (state = {}, keyPath, fn, defaultValue) => {
    if (typeof keyPath === 'string' || typeof keyPath === 'number') {
        keyPath = [keyPath];
    }
    const key = keyPath.shift();
    let subState = state[key] ?? {};
    if (keyPath.length === 0) {
        try {
            return fn(state[key]);
        }
        catch (e) {
            return defaultValue;
        }
    }
    if (typeof subState !== 'object') {
        subState = {};
    }
    return getStateIn(subState, keyPath, fn, defaultValue);
};
export const updateState = (state = {}, keyPath, value) => {
    if (typeof keyPath === 'string' || typeof keyPath === 'number') {
        keyPath = [keyPath];
    }
    if (keyPath.length === 0) {
        return value;
    }
    const key = keyPath.shift();
    let subState = state[key] ?? {};
    if (typeof subState !== 'object') {
        subState = {};
    }
    return {
        ...state,
        [key]: updateState(subState, keyPath, value),
    };
};
export const updateStateIn = (state = {}, keyPath, fn) => {
    if (typeof keyPath === 'string' || typeof keyPath === 'number') {
        keyPath = [keyPath];
    }
    const key = keyPath.shift();
    if (keyPath.length === 0) {
        try {
            return {
                ...state,
                [key]: fn(state[key]),
            };
        }
        catch (e) {
            return state;
        }
    }
    let subState = state[key] ?? {};
    if (typeof subState !== 'object') {
        subState = {};
    }
    return {
        ...state,
        [key]: updateStateIn(subState, keyPath, fn),
    };
};
export const removeState = (state = {}, keyPath) => {
    if (typeof keyPath === 'string' || typeof keyPath === 'number') {
        keyPath = [keyPath];
    }
    const key = keyPath.shift();
    if (keyPath.length === 0) {
        return Object.keys(state).reduce((m, n) => {
            if (n === key)
                return m;
            m[n] = state[n];
            return m;
        }, {});
    }
    const subState = removeState((state[key] ?? {}), keyPath);
    // remove key if it doesnot has any child
    if (Object.keys(subState).length === 0) {
        return Object.keys(state).reduce((m, n) => {
            if (n === key)
                return m;
            m[n] = state[n];
            return m;
        }, {});
    }
    return {
        ...state,
        [key]: subState,
    };
};
